package cn.com.qmth.scancentral.service.impl;

import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.qmth.boot.core.exception.ParameterException;
import com.qmth.boot.core.security.annotation.AuthorizationComponent;
import com.qmth.boot.core.security.service.AuthorizationService;
import com.qmth.boot.tools.signature.SignatureType;

import cn.com.qmth.scancentral.bean.ScannerLoginInfo;
import cn.com.qmth.scancentral.bean.User;
import cn.com.qmth.scancentral.entity.SystemConfigEntity;
import cn.com.qmth.scancentral.entity.UserEntity;
import cn.com.qmth.scancentral.enums.Role;
import cn.com.qmth.scancentral.exception.ForbiddenExceptions;
import cn.com.qmth.scancentral.exception.UnauthorizedExceptions;
import cn.com.qmth.scancentral.service.AuthService;
import cn.com.qmth.scancentral.service.ScannerService;
import cn.com.qmth.scancentral.service.SessionService;
import cn.com.qmth.scancentral.service.UserService;

@Service
@AuthorizationComponent
public class AuthServiceImpl implements AuthorizationService<User>, AuthService {

    protected static final Logger log = LoggerFactory.getLogger(AuthServiceImpl.class);

    @Autowired
    private UserService userService;

    @Autowired
    private SessionService sessionService;

    @Autowired
    private ScannerService scannerService;

    @Autowired
    private SystemConfigServiceImpl systemConfigService;

    @Override
    public User findByIdentity(String identity, SignatureType type, String path) {
        return sessionService.getSessionUser(identity);
    }

    @Override
    public boolean hasPermission(User user, String path) {
        sessionService.updateUserSession(user);
        return true;
    }

    @Transactional
    @Override
    public User login(String ip, ScannerLoginInfo loginInfo) {
        if (StringUtils.isBlank(ip)) {
            throw new ParameterException("IP获取失败");
        }
        SystemConfigEntity config = systemConfigService.find();
        if (config != null) {
            if (config.getScannerEnableLogin() != null && !config.getScannerEnableLogin()) {
                throw ForbiddenExceptions.SCANNER_LOGIN_DISABLE;
            }
            if (StringUtils.isNotEmpty(config.getScannerPassword())) {
                if (loginInfo.getPassword() == null) {
                    throw UnauthorizedExceptions.SCANNER_PASSWORD_INVALID;
                }
                if (!config.getScannerPassword().equals(loginInfo.getPassword())) {
                    throw UnauthorizedExceptions.SCANNER_PASSWORD_INVALID;
                }
            }
        }
        User user = new User();
        user.setRole(Role.SCANNER);
        user.setAccount(ip);
        user.setAccessToken(RandomStringUtils.randomAlphanumeric(32));
        user.buildKey();
        scannerService.onLogin(ip);
        sessionService.userLogin(user);
        return user;
    }

    @Override
    public User adminLogin(String loginName, String password, String device) {
        UserEntity userEntity = userService.getByLoginName(loginName);
        if (userEntity == null) {
            throw new ParameterException("用户不存在");
        }
        if (!userEntity.getEnable()) {
            throw new ParameterException("用户已禁用");
        }
        if (!password.equals(userEntity.getPassword())) {
            throw new ParameterException("密码错误");
        }
        userEntity.setDevice(device);
        userService.saveOrUpdate(userEntity);
        User user = new User();
        user.setRole(userEntity.getRole());
        user.setAccount(loginName);
        user.setAccessToken(RandomStringUtils.randomAlphanumeric(32));
        user.buildKey();
        user.setId(userEntity.getId());
        user.setName(userEntity.getName());
        sessionService.userLogin(user);
        return user;
    }

}
